﻿using System;
using System.Windows.Forms;
using ThinkGeo.MapSuite;
using ThinkGeo.MapSuite.Drawing;
using ThinkGeo.MapSuite.Layers;
using ThinkGeo.MapSuite.Shapes;
using ThinkGeo.MapSuite.Styles;
using ThinkGeo.MapSuite.WinForms;

namespace _9119
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            winformsMap1.MapUnit = GeographyUnit.DecimalDegree;

            winformsMap1.TrackOverlay = new MyTrackInteractiveOverlay();
            winformsMap1.ExtentOverlay = new MyExtentOverlay();
            
            LayerOverlay layerOverlay = new LayerOverlay();
            ShapeFileFeatureLayer shapeFileLayer = new ShapeFileFeatureLayer(@"..\..\AppData\states.shp");
            shapeFileLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = WorldStreetsAreaStyles.Military();
            shapeFileLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            layerOverlay.Layers.Add(shapeFileLayer);
            winformsMap1.Overlays.Add(layerOverlay);

            winformsMap1.TrackOverlay.TrackMode = TrackMode.Line;

            ValueStyle style = new ValueStyle();
            style.ColumnName = "style";
            style.ValueItems.Add(new ValueItem("start", PointStyles.CreateSimpleStarStyle(GeoColors.Green, 20)));
            style.ValueItems.Add(new ValueItem("end", PointStyles.CreateSimplePointStyle(PointSymbolType.Diamond, GeoColors.Red, 20)));

            winformsMap1.TrackOverlay.TrackShapeLayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(style);
            winformsMap1.TrackOverlay.TrackShapeLayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(winformsMap1.TrackOverlay.TrackShapeLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle);
            winformsMap1.TrackOverlay.TrackShapeLayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(winformsMap1.TrackOverlay.TrackShapeLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle);
            winformsMap1.TrackOverlay.TrackShapeLayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(winformsMap1.TrackOverlay.TrackShapeLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle);


            winformsMap1.TrackOverlay.TrackShapeLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = null;
            winformsMap1.TrackOverlay.TrackShapeLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = null;
            winformsMap1.TrackOverlay.TrackShapeLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = null;
            winformsMap1.TrackOverlay.TrackShapeLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = null;

            winformsMap1.TrackOverlay.TrackShapeLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;


            winformsMap1.TrackOverlay.TrackStarting += TrackOverlay_TrackStarting;
            winformsMap1.TrackOverlay.TrackEnded += TrackOverlay_TrackEnded;

            shapeFileLayer.Open();
            winformsMap1.CurrentExtent = shapeFileLayer.GetBoundingBox();
            winformsMap1.Refresh();
        }
        
        private void TrackOverlay_TrackEnded(object sender, TrackEndedTrackInteractiveOverlayEventArgs e)
        {
            if (e.TrackShape is LineShape)
            {
                LineShape line = e.TrackShape as LineShape;
                Feature endPoint = new Feature(line.Vertices[line.Vertices.Count - 1]);
                endPoint.ColumnValues.Add("style", "end");
                winformsMap1.TrackOverlay.TrackShapeLayer.InternalFeatures.Add(endPoint);
            }
        }

        private void TrackOverlay_TrackStarting(object sender, TrackStartingTrackInteractiveOverlayEventArgs e)
        {
            if (winformsMap1.TrackOverlay.TrackMode == TrackMode.Line)
            {
                Feature startPoint = new Feature(e.StartingVertex);
                startPoint.ColumnValues.Add("style", "start");
                winformsMap1.TrackOverlay.TrackShapeLayer.InternalFeatures.Add(startPoint);
            }
        }
    }


    public class MyTrackInteractiveOverlay : TrackInteractiveOverlay
    {
        protected override InteractiveResult MouseMoveCore(InteractionArguments interactionArguments)
        {
            InteractiveResult interactiveResult = new InteractiveResult();

            if (interactionArguments.MouseButton == MapMouseButton.None)
            {
                interactiveResult = base.MouseMoveCore(interactionArguments);
            }

            interactiveResult.ProcessOtherOverlaysMode = ProcessOtherOverlaysMode.ProcessOtherOverlays;

            return interactiveResult;
        }

        protected override InteractiveResult MouseDownCore(InteractionArguments interactionArguments)
        {
            InteractiveResult interactiveResult = new InteractiveResult();

            if (interactionArguments.MouseButton == MapMouseButton.Left)
            {
                return base.MouseDownCore(interactionArguments);
            }

            return interactiveResult;
        }

        protected override InteractiveResult MouseUpCore(InteractionArguments interactionArguments)
        {
            InteractiveResult interactiveResult = new InteractiveResult();

            if (interactionArguments.MouseButton == MapMouseButton.Left)
            {
                return base.MouseUpCore(interactionArguments);
            }

            return interactiveResult;
        }

        protected override InteractiveResult MouseDoubleClickCore(InteractionArguments interactionArguments)
        {
            InteractiveResult interactiveResult = new InteractiveResult();

            if (interactionArguments.MouseButton == MapMouseButton.Left)
            {
                return base.MouseDoubleClickCore(interactionArguments);
            }

            return interactiveResult;
        }
    }

    public class MyExtentOverlay : ExtentInteractiveOverlay
    {
        protected override InteractiveResult MouseDownCore(InteractionArguments interactionArguments)
        {
            if (interactionArguments.MouseButton == MapMouseButton.Right)
            {
                interactionArguments.MouseButton = MapMouseButton.Left;
            }

            return base.MouseDownCore(interactionArguments);
        }

        protected override InteractiveResult MouseMoveCore(InteractionArguments interactionArguments)
        {
            if (interactionArguments.MouseButton == MapMouseButton.Right)
            {
                interactionArguments.MouseButton = MapMouseButton.Left;
            }

            return base.MouseMoveCore(interactionArguments);
        }

        protected override InteractiveResult MouseUpCore(InteractionArguments interactionArguments)
        {
            if (interactionArguments.MouseButton == MapMouseButton.Right)
            {
                interactionArguments.MouseButton = MapMouseButton.Left;
            }

            return base.MouseUpCore(interactionArguments);
        }
    }
}
